%% Analysis script

for i = 1:length(MarkerNames)
    MarkerNames{i,1}(strfind(MarkerNames{i,1}, '=')) = [];
    MarkerNames{i,1} = MarkerNames{i,1}(find(~isspace(MarkerNames{i,1})));
end
MarkerNames{(length(MarkerNames)+1),1} = 'ID';
FlowJo_markers = MarkerNames';

% creates a structure array for each marker with its values as a "1xn double matrix"
for iter=1:length(FlowJo_markers)
    MarkerStruct.(FlowJo_markers{iter})=FlowJo_matrix(:,iter);
end

% creates "AllMarkers", a matrix consisting of logical values (0,1)
% according to the thresholds set before (1 = value above threshold, 0 =
% value below threshold)

for vari = 1:length(FlowJo_markers)
    Real_MarkerStruct{vari,1} = FlowJo_markers(:,vari);
    Real_MarkerStruct{vari,2} = FlowJo_matrix(:,vari);
end

logical_allMarkers = cell(length(Real_MarkerStruct),1); % preallocate

for var = 1:length(allthresholds)
    RMS = Real_MarkerStruct{var,2};
    RMSl = RMS > allthresholds(var);
    RMSl_matrix = double(RMSl);
    logical_allMarkers{var} = RMSl_matrix;
end

AllMarkers = cell2mat(logical_allMarkers');


% UniqueComb gives you the combinations present only once in the data set
[UniqueComb] = unique(AllMarkers,'rows');
SizeUC = size(UniqueComb);
NumberofUniqueComb = SizeUC(1);
mt = 1:NumberofUniqueComb;

for hoho = 1:mt
    Phenotype(hoho).index = UniqueComb(hoho,:);
end

% Saves in "CellA" all logical vectors where 1s represent the matches of
% combinations in csv file with all possible marker combinations

for i = mt;
   CellA{i} = ismember(AllMarkers,UniqueComb(i,:),'rows')';
end

% saves in "CellB" the number of hits i.e. marker combinations on cells
for j = mt
   CellB{j} = sum(CellA{1, j});
end


% finds the indices of hits for each phenotype present in the dataset and
% saves them into Indices(1)...Indices(n)

for kae = 1:NumberofUniqueComb
    Indices(kae).index = find(CellA{1,kae}(1,:));
end

% goes through all the phenotypes existing in the dataset and returns a
% cell array in which for the phenotypes (as ordered from top to bottom as
% in "UniqueComb") their ID in Imaris is saved

for hu = 1:NumberofUniqueComb
    for hex = 1:length(Indices(hu).index)
        IDs_Phenotypes{hu}(hex).index = Real_MarkerStruct{(1+length(allthresholds)),2}((Indices(hu).index(1,hex)),1);
    end
end

for he = 1:NumberofUniqueComb
a{he} = struct2cell(IDs_Phenotypes{1,he});
aforexp{he} = a{1,he,1}(:,:); 
end

for ha = 1:NumberofUniqueComb
    IDs(ha).data = cell2mat(aforexp{1,ha}');
end

for ka = 1:NumberofUniqueComb
    ag{ka} = length(IDs(ka).data);
end

aha = cell2mat(ag);
MaxVal = max(aha);

for bu = 1:NumberofUniqueComb
    if MaxVal > length(IDs(bu).data)
        IDs(bu).data(end+1:MaxVal) = NaN;
    end
end

for bla = 1:NumberofUniqueComb
    p = size(IDs(bla).data);
    if p(1) == 1
        IDs(bla).data = IDs(bla).data';
    end
end

for blo = 1:NumberofUniqueComb
    AaA{blo} = IDs(blo).data;
end


% transfers the absolute numbers of hits from a cell array to a vector that
% displays the hits as percentages of total hits
number_of_cells = CellB';
Vector_CombHits = cell2mat(number_of_cells);
sumofallhits = sum(Vector_CombHits);
Percentage_of_total_cells = Vector_CombHits*((1/sumofallhits)*100);

% creates a cell array that contains only the total number of cells in the
% first row, all other rows are empty
total_cells = {sumofallhits};
total_cells{NumberofUniqueComb, 1} = [];

% Compares hits (as %) to entered value of false positive and sets
% everything below to 0

for i = mt
    if Percentage_of_total_cells(i) < falsepositive;
        Percentage_of_total_cells(i) = 0;
        ind{i,1} = i; 
    else Percentage_of_total_cells;
    end
end

% IDsforExport are the ID numbers of the cells within the subpopulations
% identified
if falsepositive > 0
    AaA(cell2mat(ind)) = [];
    IDs(cell2mat(ind)) = [];
end

IDsforExport = cell2mat(AaA);
Cell_IDs = num2cell(IDsforExport);
Cell_IDs(cellfun(@isnan,Cell_IDs)) = {[]};
ID_table = cell2table(Cell_IDs');
%% 
% extracts the Percentage of the parentpopulation and reduces the data set
% to the original size
UniqueComb_1= [UniqueComb Percentage_of_total_cells];

extracted_percentages1 = UniqueComb_1(:,(1+length(allthresholds)));
UniqueComb_1_raw = UniqueComb_1;

toDelete3 = UniqueComb_1(:,(1+length(allthresholds))) == 0;
UniqueComb_1(toDelete3,:) = [];

Percentage_of_parentpopulation = UniqueComb_1(:,(1+length(allthresholds)));
UniqueComb_1_abovefalsepositive = UniqueComb_1;

UniqueComb_1(:,(1+length(allthresholds))) = [];



% replaces the "hits" (= 1s) by the marker name of that specific column
UniqueComb_logi = logical(UniqueComb_1);
[Uv, Uuv] = find(UniqueComb_1);
Uuuv = [Uv Uuv];
ra = num2cell(Uuuv);
ras = num2cell(UniqueComb_logi);
Sr = size(ras);
Sr1 = Sr(1);
marker = cellfun(@(ras)ras(logical(ras)),ras,'uni',false);


for varia = 1:size(UniqueComb_1,2)
	for variab = 1:Sr1
    	if ~isempty(marker{variab,varia})
        	marker{variab,varia} = Real_MarkerStruct{varia,1}{1,1};
      	elseif isempty(marker{variab,varia})
          	marker{variab,varia} = ' ';
        end
 	end
end

%% Table Creator Pop Analysis
% creates a table of the combinations that are present in the sample , 
% total cells per combinations and percentages (of whole number of 
% combinations); creates also an excel file where each column represents a
% subpopulation and write for those the Imaris ID number; creates .txt
% files for every subpopulation with the IDs stored as comma separated list
% for pasting in Imaris and thus creating a new isosurface

Marker_Table = cell2table(marker);
ID = zeros(length(UniqueComb_1),1);
ID = num2cell(ID);

for kop = 1:length(UniqueComb_1)   
        ID{kop,1} = '-';
end

Spacer = cell2table(ID);

extracted_percentages1_1_Table = array2table(Percentage_of_parentpopulation);

if falsepositive > 0 
    number_of_cells(cell2mat(ind)) = [];
end
numbertable = cell2table(number_of_cells);

% Excel files with IDs for all subpops in columns
for t = 1:size(marker,1)
    str{1,t} = strjoin(marker(t,:));
end
marker_for_plot{kkt,1} = marker;

dat = [str; num2cell(IDsforExport)];
gname = [num2str(kkt) '_IDs.xlsx'];
FileName = fullfile(currentFolder, theName, gname);
xlswrite(FileName,dat)
clearvars Filename

% Txt file with IDs for parentpop to have the cropped pop IDs
Tname =  'ParentPop_IDs.txt';
FileName = fullfile(currentFolder, theName, Tname);
dlmwrite(FileName,Real_MarkerStruct{(1+length(allthresholds)),2}','delimiter',',')
clearvars thisname str FileName


% Txt file(s) with IDs as comma separated list for pasting in Imaris
tra_Cell_IDs = Cell_IDs';
for i = 1:size(tra_Cell_IDs,1)
    identifier = num2str(kkt);
    str = strjoin(marker(i,:));
    thisname = [identifier '_' str '_IDs.txt'];
    FileName = fullfile(currentFolder, theName, thisname);
    dlmwrite(FileName,tra_Cell_IDs(i,:),'delimiter',',')
    clearvars thisname str FileName
end

% Excel File with marker combinations, percentages and total number of
% cells
Super_ID_table = [Marker_Table extracted_percentages1_1_Table numbertable];
FileName = fullfile(currentFolder, theName, woi);
writetable(Super_ID_table, FileName,'Delimiter',';')
clearvars Filename


%% Data extraction from stats

for i = 1:length(IDs)
    [d{i}, dd, ddd{i}] = intersect(IDs(i).data,Stats(:,(size(Stats,2))));
end

for i = 1:length(IDs)
    for j = 1:(size(Stats,2)-1)
    ExSt{j} = Stats(ddd{i},j);
    end
    ExSt{1,j+1} = d{i};
    ExcSta = cell2mat(ExSt);
    ExctStat{1,i} = ExcSta(:,1:end);
end


% write for every subpopulation the stats + XYZ coord. to an excel file
for i = 1:length(IDs)
    
    s = [StatNames'; num2cell(ExctStat{1,i})];
    
    Name = ['Stats_' Marker_Table{i,:}];
    
    for j = 1:length(Name)
        Name1(1,j) = ischar(Name{1,j});
    end
    Name2 = find(Name1);
    Zeros = find(Name1 == 0);
    Name(Zeros) = {'neg_'};
    Name3 = cell2mat(Name(Name2));
    Name4 =[num2str(kkt) '_' Name3 '.xlsx'];
   
    FileName = fullfile(currentFolder, theName, Name4);
    xlswrite(FileName,s)
    clearvars Name Name1 Name2 Name3 Name4 Zeros s FileName
end
